/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2008  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS version: 3.62c                                         *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c (for Freescale Coldfire and IAR)
Purpose : LED interface for M52223EVB evaluation board
--------  END-OF-HEADER  ---------------------------------------------
*/

#define BSP_C
#include "BSP.h"


/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

#define PTFD             (*(volatile unsigned char*) 0xFFFF800A)
#define PTFDD            (*(volatile unsigned char*) 0xFFFF800b)


/****** Assign LEDs to Ports ****************************************/
#define LED0_BIT         (0)
#define LED1_BIT         (1)

#define LEDPORT_DATA     PTFD
#define LEDPORT_DIR      PTFDD

#define LEDMASK_0        (1uL << LED0_BIT)
#define LEDMASK_1        (1uL << LED1_BIT)
#define LEDMASK_ALL      (LEDMASK_0 | LEDMASK_1)

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       LED_Init()
*/
void BSP_Init(void) {
  LEDPORT_DIR  |= LEDMASK_ALL;  // Switch LED-Ports to output mode
  LEDPORT_DATA |= LEDMASK_ALL;  // Initially switch off LEDs
}

/*********************************************************************
*
*       LED switching routines
*       LEDs are switched on with high level on port lines
*/
void BSP_SetLED(int Index) {
  if (Index == 0) {
    LEDPORT_DATA &= ~(LEDMASK_0); /* Switch on LED 0     */
  } else {
    LEDPORT_DATA &= ~(LEDMASK_1); /* Switch on LED 1     */
  }
}

void BSP_ClrLED(int Index) {
  if (Index == 0) {
    LEDPORT_DATA |= (LEDMASK_0); /* Switch off LED 0     */
  } else {
    LEDPORT_DATA |= (LEDMASK_1); /* Switch off LED 1     */
  }
}

void BSP_ToggleLED(int Index) {
  if (Index == 0) {
    PTFD ^= (1uL << LED0_BIT);
  } else {
    PTFD ^= (1uL << LED1_BIT);
  }
}

/****** End of file *************************************************/
