/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2008  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS version: 3.62c                                         *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : RTOSINIT_MC51AC256.c
          for Freescale V1 (MC51AC256)

Purpose : Initializes and handles the hardware for the OS as far
          as required by the OS.
          Feel free to modify this file acc. to your
          target system.

--------  END-OF-HEADER  ---------------------------------------------
*/

#include "RTOS.h"

/*********************************************************************
*
*       Configuration
*
**********************************************************************
*/

#define ALLOW_NESTED_INTERRUPTS  (0)  // Caution: Nesting interrupts will cause higher stack load on system stack CSTACK and tsak stacks

/*********************************************************************
*
*       Clock frequency settings
*/
#define OS_FSYS (48000000L)           // Assuming PLL is initialized, external crystal 4MHz

#ifndef   OS_TICK_FREQ
  #define OS_TICK_FREQ (1000)
#endif

#ifndef   OS_PCLK_TIMER
  #define OS_PCLK_TIMER  (OS_FSYS/2)
#endif

#ifndef   OS_INIT_PLL
  #define OS_INIT_PLL (1)
#endif

/*********************************************************************
*
*       Configuration of communication to OSView
*/
#ifndef   OS_VIEW_ENABLE            // Global enable of communication
  #define OS_VIEW_ENABLE    (0)     // Default: off, uart is not tested because DEMOEM eval board
#endif                              // has no dsub9 connector

/*********************************************************************
*
*       UART settings for OSView
*       If you do not want (or can not due to hardware limitations)
*       to dedicate a UART to OSView, please define it to be -1
*       Currently UART support is not available for the MCF51AC256
*/
#ifndef   OS_UART
  #define OS_UART (0)
#endif

#ifndef   OS_BAUDRATE
  #define OS_BAUDRATE (38400)
#endif

#ifndef   OS_PCLK_UART
  #define OS_PCLK_UART  (OS_FSYS/2)
#endif

/*********************************************************************
*
* Configure interrupt vector table location
*/
#ifndef    OS_ISRHANDLERTABLE_ROM
  #define  OS_ISRHANDLERTABLE_ROM (0)
#endif

/****** End of configuration settings *******************************/

/*********************************************************************
*
*       UART settings for OSView
*
*       Automatically generated from configuration settings
*/
#define OS_UART_USED   ((OS_VIEW_ENABLE != 0) && (0))

/*********************************************************************
*
*       Local defines (sfrs used in RTOSInit.c)
*
**********************************************************************
*/
#define NUM_INT_SOURCES           56

#define SOPT1         (*(volatile OS_U8*) (0xFFFF9802))  // System Options register

#define TPM_BASE_ADDR (0xFFFF9830)
#define TPMSC         (*(volatile OS_U8*) (TPM_BASE_ADDR + 0x00))  // Timer status and Control register
#define TPMCNT        (*(volatile OS_U16*)(TPM_BASE_ADDR + 0x01))  // Timer counter register
#define TPMMOD        (*(volatile OS_U16*)(TPM_BASE_ADDR + 0x03))  // Timer counter modulo register
#define TPMC0SC       (*(volatile OS_U8*) (TPM_BASE_ADDR + 0x05))  // Timer channel 0 status and control register
#define TPMC0V        (*(volatile OS_U16*)(TPM_BASE_ADDR + 0x06))  // Timer channel 0 value register
#define TPMC0SC_CH0F  (1uL << 7)

#define SMCLK         (*(volatile OS_U8*) (0xFFFF9803))
#define SPMSC1        (*(volatile OS_U8*) (0xFFFF9809))
#define SPMSC2        (*(volatile OS_U8*) (0xFFFF980A))

#define MCGC1         (*(volatile OS_U8*) (0xFFFF8048))
#define MCGC2         (*(volatile OS_U8*) (0xFFFF8049))
#define MCGC3         (*(volatile OS_U8*) (0xFFFF804C))
#define MCGC4         (*(volatile OS_U8*) (0xFFFF804D))
#define MCGSC         (*(volatile OS_U8*) (0xFFFF804B))

#define MCGSC_OSCINIT_BIT   (1)
#define MCGSC_IREFST_BIT    (4)
#define MCGSC_PLLST_BIT     (5)
#define MCGSC_LOCK_BIT      (6)
#define MCGSC_IREFST_MASK   (1uL << MCGSC_IREFST_BIT)
#define MCGSC_PLLST_MASK    (1uL << MCGSC_PLLST_BIT)
#define MCGSC_LOCK_MASK     (1uL << MCGSC_LOCK_BIT)
#define MCGSC_OSCINIT_MASK  (1uL << MCGSC_OSCINIT_BIT)

#if OS_UART_USED
  #define OS_COM_INIT() OS_COM_Init()
#else
  #define OS_COM_INIT()
#endif

/*********************************************************************
*
*       Prototypes
*
**********************************************************************
*/
void _OS_IrqDefaultHandler(void);
void _OS_SystemIrqhandler(void);

/*********************************************************************
*
*       Externals
*
**********************************************************************
*/

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/
#if (OS_ISRHANDLERTABLE_ROM == 1)
static OS_ISR_HANDLER* const _papISR[NUM_INT_SOURCES] =
#else
static OS_ISR_HANDLER* _papISR[NUM_INT_SOURCES] =
#endif
{
  _OS_IrqDefaultHandler,                         // Handler for interrupt 64
  _OS_IrqDefaultHandler,                         // Handler for interrupt 65
  _OS_IrqDefaultHandler,                         // Handler for interrupt 66
  _OS_IrqDefaultHandler,                         // Handler for interrupt 67
  _OS_IrqDefaultHandler,                         // Handler for interrupt 68
  _OS_IrqDefaultHandler,                         // Handler for interrupt 69
  _OS_IrqDefaultHandler,                         // Handler for interrupt 70
  _OS_IrqDefaultHandler,                         // Handler for interrupt 71
  _OS_IrqDefaultHandler,                         // Handler for interrupt 72
  _OS_IrqDefaultHandler,                         // Handler for interrupt 73
  _OS_IrqDefaultHandler,                         // Handler for interrupt 74
  _OS_IrqDefaultHandler,                         // Handler for interrupt 75
  _OS_IrqDefaultHandler,                         // Handler for interrupt 76
  _OS_IrqDefaultHandler,                         // Handler for interrupt 77
  _OS_IrqDefaultHandler,                         // Handler for interrupt 78
  _OS_IrqDefaultHandler,                         // Handler for interrupt 79
  _OS_IrqDefaultHandler,                         // Handler for interrupt 80
  _OS_IrqDefaultHandler,                         // Handler for interrupt 81
  _OS_IrqDefaultHandler,                         // Handler for interrupt 82
  _OS_IrqDefaultHandler,                         // Handler for interrupt 83
  _OS_IrqDefaultHandler,                         // Handler for interrupt 84
  _OS_IrqDefaultHandler,                         // Handler for interrupt 85
  _OS_IrqDefaultHandler,                         // Handler for interrupt 86
  _OS_IrqDefaultHandler,                         // Handler for interrupt 87
  _OS_IrqDefaultHandler,                         // Handler for interrupt 88
  _OS_IrqDefaultHandler,                         // Handler for interrupt 89
  _OS_IrqDefaultHandler,                         // Handler for interrupt 90
  _OS_IrqDefaultHandler,                         // Handler for interrupt 91
  _OS_SystemIrqhandler,                          // Handler for interrupt 92, embOS system timer
  _OS_IrqDefaultHandler,                         // Handler for interrupt 93
  _OS_IrqDefaultHandler,                         // Handler for interrupt 94
  _OS_IrqDefaultHandler,                         // Handler for interrupt 95
  _OS_IrqDefaultHandler,                         // Handler for interrupt 96
  _OS_IrqDefaultHandler,                         // Handler for interrupt 97
  _OS_IrqDefaultHandler,                         // Handler for interrupt 98
  _OS_IrqDefaultHandler,                         // Handler for interrupt 99
  _OS_IrqDefaultHandler,                         // Handler for interrupt 100
  _OS_IrqDefaultHandler,                         // Handler for interrupt 101
  _OS_IrqDefaultHandler,                         // Handler for interrupt 102
  _OS_IrqDefaultHandler,                         // Handler for interrupt 103
  _OS_IrqDefaultHandler,                         // Handler for interrupt 104
  _OS_IrqDefaultHandler,                         // Handler for interrupt 105
  _OS_IrqDefaultHandler,                         // Handler for interrupt 106
  _OS_IrqDefaultHandler,                         // Handler for interrupt 107
  _OS_IrqDefaultHandler,                         // Handler for interrupt 108
  _OS_IrqDefaultHandler,                         // Handler for interrupt 109
  _OS_IrqDefaultHandler,                         // Handler for interrupt 110
  _OS_IrqDefaultHandler,                         // Handler for interrupt 111
  _OS_IrqDefaultHandler,                         // Handler for interrupt 112
  _OS_IrqDefaultHandler,                         // Handler for interrupt 113
  _OS_IrqDefaultHandler,                         // Handler for interrupt 114
  _OS_IrqDefaultHandler,                         // Handler for interrupt 115
  _OS_IrqDefaultHandler,                         // Handler for interrupt 116
  _OS_IrqDefaultHandler,                         // Handler for interrupt 117
  _OS_IrqDefaultHandler,                         // Handler for interrupt 118
  _OS_IrqDefaultHandler                          // Handler for interrupt 119
};

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

/*********************************************************************
*
*        _OS_SystemIrqhandler
*/
void _OS_SystemIrqhandler(void) {
  TPMC0SC &= ~TPMC0SC_CH0F;  // Reset compare interrupt request flag, TPM1C0SC: CH0F=0
  OS_HandleTick();
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_InitHW()
*
*       Initialize the hardware (timer) required for the OS to run.
*       May be modified, if an other timer should be used
*/
#define OS_TIMER_RELOAD (OS_PCLK_TIMER/OS_TICK_FREQ)
#if (OS_TIMER_RELOAD >= 0x10000)
  #error "TPM1 timer can not be used, please check configuration"
#endif

void OS_InitHW(void) {
  int i;

  OS_IncDI();
  //
  //  Disable Watchdog, allow wait mode
  //
  SOPT1 = 0x50;
  //
  // Initiliaze interrupt vector table in RAM if required
  //
#if (OS_ISRHANDLERTABLE_ROM == 0)
  for (i = 0; i < NUM_INT_SOURCES; +i++) {
    _papISR[i] = (OS_ISR_HANDLER*)_OS_IrqDefaultHandler;
  }
#endif

#if OS_INIT_PLL
  //
  //  Initialize oscillator and PLL
  //
  /* SPMSC1: LVDF=0,LVDACK=0,LVDIE=0,LVDRE=1,LVDSE=1,LVDE=1,??=0,BGBE=0 */
  SPMSC1 =  0x1C;
  /* SPMSC2: LVWF=0,LVWACK=0,LVDV=0,LVWV=0,PPDF=0,PPDACK=0,??=0,PPDC=0 */
  SPMSC2 =  0x00;
  /* SMCLK: MPE=0,MCSEL=0 */
  SMCLK &= ~0x17uL;
  /* MCGC2: BDIV=0,RANGE=1,HGO=0,LP=0,EREFS=1,ERCLKEN=1,EREFSTEN=0 */
  MCGC2  = 0x26;
  /* MCGC3: DIV32=1 */
  MCGC3 |= 0x10;
  /* MCGC1: CLKS=2,RDIV=2,IREFS=0,IRCLKEN=1,IREFSTEN=0 */
  MCGC1  = 0x92;
  /* MCGC3: LOLIE=0,PLLS=0,CME=0,DIV32=1,VDIV=6 */
  MCGC3  = 0x16;
  /* MCGC4: ??=0,??=0,DMX32=0,??=0,??=0,??=0,DRST_DRS=0 */
  MCGC4  = 0x00;
  while((MCGSC & MCGSC_OSCINIT_MASK) == 0) {  // Wait until external clock is stable
  }
  while(MCGSC & MCGSC_IREFST_MASK) {          // Wait until external clock is selected
  }
  while((MCGSC & 0x0C) != 0x08) {             // Wait until external clock is selected as a bus clock
  }
  /* MCGC2: BDIV=0,RANGE=1,HGO=0,LP=1,EREFS=1,ERCLKEN=1,EREFSTEN=0 */
  MCGC2  = 0x2E;
  /* MCGC1: CLKS=2,RDIV=1,IREFS=0,IRCLKEN=1,IREFSTEN=0 */
  MCGC1  = 0x8A;
  /* MCGC3: LOLIE=0,PLLS=0,CME=0,DIV32=0,VDIV=6 */
  MCGC3  = 0x06;
  /* MCGC3: LOLIE=0,PLLS=1,CME=0,DIV32=0,VDIV=6 */
  MCGC3  = 0x46;
  while((MCGSC & MCGSC_PLLST_MASK) == 0) {     // Wait until PLL is selected
  }
  /* MCGC2: LP=0 */
  MCGC2 &= ~0x08uL;
  while((MCGSC & MCGSC_LOCK_MASK) == 0) {      // Wait until PLL is locked
  }
  /* MCGC1: CLKS=0,RDIV=1,IREFS=0,IRCLKEN=1,IREFSTEN=0 */
  MCGC1  = 0x0A;
  while((MCGSC & 0x0C) != 0x0C) {      // Wait until PLL clock is selected as a bus clock
  }
#endif
  //
  // Initialize embOS timer
  //
  TPMSC    = 0x00;               // Stop HW; disable overflow interrupt and set prescaler to 0
  TPMC0SC  = 0x50;               // Set output compare mode and enable compare interrupt
  TPMC0V   = (OS_TIMER_RELOAD);  // Initialize appropriate value to the compare/modulo/reload register
  TPMMOD   = (OS_TIMER_RELOAD);
  TPMCNT   = 0x00;               // Reset HW Counter
  TPMSC    = 0x08;               // No prescaler, select BusClk and run timer
  OS_EnableISR(28, _OS_SystemIrqhandler);

  OS_COM_INIT();
  OS_DecRI();
}

/*********************************************************************
*
*       Idle loop  (OS_Idle)
*
*       Please note:
*       This is basically the "core" of the idle loop.
*       This core loop can be changed, but:
*       The idle loop does not have a stack of its own, therefore no
*       functionality should be implemented that relies on the stack
*       to be preserved. However, a simple program loop can be programmed
*       (like toggeling an output or incrementing a counter)
*/
void OS_Idle(void) {           // Idle loop: No task is ready to exec
  while (1) {
    //
    // Some NOPs may be necessary for bdm debugger
    // to prevent bdm debugger lock-up
    //
    #ifdef __IAR_SYSTEMS_ICC__
      __no_operation();
      __no_operation();
    #else
      asm(NOP);
      asm(NOP);
    #endif
  }
}

/*********************************************************************
*
*       Get time [cycles]
*
*       This routine is required for task-info via OSView or high
*       resolution time maesurement functions.
*       It returns the system time in timer clock cycles.
*/
OS_U32 OS_GetTime_Cycles(void) {
  OS_U32 Time;
  OS_U32 Cnt;

  Time = (OS_U32)OS_GetTime32();
  Cnt  = TPMCNT;
  //
  // Check if timer interrupt pending ...
  //
  if (TPMC0SC & TPMC0SC_CH0F) {  // Adjust result
    Cnt = TPMCNT;
    Time++;
  }
  return (OS_TIMER_RELOAD * Time) + Cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us
*
*       Convert Cycles into micro seconds.
*
*       If your clock frequency is not a multiple of 1 MHz,
*       you may have to modify this routine in order to get proper
*       diagonstics.
*
*       This routine is required for profiling or high resolution time
*       measurement only. It does not affect operation of the OS.
*/
OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  return Cycles/(OS_PCLK_TIMER/1000000);
}

/*********************************************************************
*
*       Communication for OSView via UART (optional)
*
**********************************************************************
*/

#if OS_UART_USED

#define BAUDDIVIDE (OS_PCLK_UART / (16 * OS_BAUDRATE))

#else

void OS_COM_Send1(OS_U8 c) {   // Dummy function, required, may be called from sources
  (void) c;                    // Avoid compiler warning
  OS_COM_ClearTxActive();      // Let the OS know that Tx is not busy
}

#endif /*  OS_UART_USED  */

/****** Final check of configuration ********************************/

#ifndef OS_UART_USED
  #error "OS_UART_USED has to be defined"
#endif

/*********************************************************************
*
*       OS interrupt handler and ISR specific functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_IrqDefaultHandler
*
*/
void _OS_IrqDefaultHandler(void) {
#if OS_DEBUG
  OS_DI();
  while (1) {
    //
    // Some NOPs may be necessary for bdm debugger
    // to prevent bdm debugger lock-up
    //
    #ifdef __IAR_SYSTEMS_ICC__
      __no_operation();
      __no_operation();
    #else
      asm(NOP);
      asm(NOP);
    #endif
  }
#endif
}

/*********************************************************************
*
*       OS_irq_handler
*
*       Detect reason for IRQ and call correspondig service routine.
*       OS_irq_handler is called from OS_ISR_Handler function
*       from the OS library
*/
void OS_irq_handler(int ISRIndex) {
  OS_ISR_HANDLER* pISR;

  #if OS_DEBUG
    if (ISRIndex < 64) {
      while(1) {
        // OS_irq_Handler handles peripheral interrupts only
        __no_operation();
        __no_operation();
      }
    }
  #endif
  pISR = *_papISR[ISRIndex - 64];
  pISR();
}

/*********************************************************************
*
*       OS_EnableISR
*/
OS_ISR_HANDLER* OS_EnableISR(int ISRIndex, OS_ISR_HANDLER* pISRHandler) {
  OS_ISR_HANDLER*  pOldHandler;
#if OS_DEBUG
  if (ISRIndex >= NUM_INT_SOURCES) {
    OS_Error(OS_ERR_ISR_INDEX);
    return NULL;
  }
#endif
  OS_DI();
  pOldHandler = _papISR[ISRIndex];       // Get old handler
#if (OS_ISRHANDLERTABLE_ROM == 0)
  _papISR[ISRIndex] = pISRHandler;       // Install new handler
#endif
  OS_RestoreI();
  return pOldHandler;
}

/*********************************************************************
*
*       OS_DisableISR
*/
void OS_DisableISR(int ISRIndex) {
#if OS_DEBUG
  if (ISRIndex >= NUM_INT_SOURCES) {
    OS_Error(OS_ERR_ISR_INDEX);
    return;
  }
#endif
  OS_DI();
#if (OS_ISRHANDLERTABLE_ROM == 0)
  _papISR[ISRIndex] = _OS_IrqDefaultHandler;   // Set default handler
#endif
  OS_RestoreI();
}

/*****  End of file  ************************************************/
