/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2008  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS version: 3.62c                                         *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c (for Freescale Coldfire and IAR)
Purpose : LED interface for M52223-SK evaluation board
--------  END-OF-HEADER  ---------------------------------------------
*/

#define BSP_C
#include "BSP.h"


/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

/****** Assign LEDs to Ports ****************************************/

#define LED0_BIT         1
#define LED1_BIT         2
#define LEDPORT_STATE    (*(volatile unsigned char*)0x4010000A)
#define LEDPORT_LATCH    (*(volatile unsigned char*)0x40100022)
#define LED_LATCH        (*(volatile unsigned char*)0x4010000A)

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       LED_Init()
*/
void BSP_Init(void) {
  LEDPORT_LATCH |= (1uL << LED0_BIT) | (1uL << LED1_BIT);
}

/*********************************************************************
*
*       LED switching routines
*       LEDs are switched on with high level on port lines
*/
void BSP_SetLED(int Index) {
  LED_LATCH |=  (1uL << (Index+1));            /* Switch on LED 0     */
}

void BSP_ClrLED(int Index) {
  LED_LATCH &= ~(1uL << (Index+1));            /* Switch off LED 0    */
}

void BSP_ToggleLED(int Index) {
  if (LEDPORT_STATE & (1uL << (Index+1))) {    /* LED is switched off */
    BSP_ClrLED(Index);
  } else {
    BSP_SetLED(Index);
  }
}

/****** EOF *********************************************************/

