/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2008  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS version: 3.62c                                         *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c (for Freescale Coldfire and IAR)
Purpose : LED interface for M52223EVB evaluation board
--------  END-OF-HEADER  ---------------------------------------------
*/

#define BSP_C
#include "BSP.h"


/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

/****** Assign LEDs to Ports ****************************************/

#define LED0_BIT         0
#define LED1_BIT         1
#define LED2_BIT         2
#define LED3_BIT         3

#define LEDPORT_STATE    (*(volatile unsigned char*)0x4010000F)
#define LEDPORT_DDR      (*(volatile unsigned char*)0x40100027)
#define LEDPORT_DATA     (*(volatile unsigned char*)0x4010000F)

#define LED_ALL_MASK     ((1uL << LED0_BIT) | (1uL << LED1_BIT) | (1uL << LED2_BIT) | (1uL << LED3_BIT))

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       LED_Init()
*/
void BSP_Init(void) {
  LEDPORT_DDR  |=  LED_ALL_MASK;  // Set data direction to output
  LEDPORT_DATA &= ~LED_ALL_MASK;  // Initially switch off LEDs

}

/*********************************************************************
*
*       LED switching routines
*       LEDs are switched on with high level on port lines
*/
void BSP_SetLED(int Index) {
  LEDPORT_DATA |= (1uL << Index);            /* Switch on LED 0     */
}

void BSP_ClrLED(int Index) {
  LEDPORT_DATA &= ~(1uL << Index);           /* Switch off LED 0    */
}

void BSP_ToggleLED(int Index) {
  if (LEDPORT_DATA & (1uL << Index)) {       /* LED is switched off */
    BSP_ClrLED(Index);
  } else {
    BSP_SetLED(Index);
  }
}

/****** End of file *************************************************/
