/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2008  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS version: 3.62c                                         *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : RTOSInit_Simulator.c                     for ColdFire CPU

Purpose : Initializes and handles the hardware for the OS as far
          as required by the OS.
          Feel free to modify this file acc. to your
          target system.
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "RTOS.h"

/*********************************************************************
*
*       Configuration
*
*********************************************************************/


/****** End of configuration settings *******************************/

/*********************************************************************
*
*       Local defines (sfrs used in RTOSInit.c)
*
**********************************************************************
*/
#define _NUM_INT_SOURCES          64


/* SFR definitions for interrupt controller */
#define ICR_BASE_ADDR  0x40000C40
#define IMRH           (*(volatile unsigned long*)0x40000C08)
#define IMRL           (*(volatile unsigned long*)0x40000C0C)

/*********************************************************************
*
*       Prototypes
*
**********************************************************************
*/
static void _OS_IrqDefaultHandler(void);

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/
#ifndef    OS_ISRHANDLERTABLE_ROM
  #define  OS_ISRHANDLERTABLE_ROM 0
#endif

#if (OS_ISRHANDLERTABLE_ROM != 0)
static OS_ISR_HANDLER* const _papISR[_NUM_INT_SOURCES] = {
  _OS_IrqDefaultHandler,      // Handler for interrupt 0
  _OS_IrqDefaultHandler,      // Handler for interrupt 1
  _OS_IrqDefaultHandler,      // Handler for interrupt 2
  _OS_IrqDefaultHandler,      // Handler for interrupt 3
  _OS_IrqDefaultHandler,      // Handler for interrupt 4
  _OS_IrqDefaultHandler,      // Handler for interrupt 5
  _OS_IrqDefaultHandler,      // Handler for interrupt 6
  _OS_IrqDefaultHandler,      // Handler for interrupt 7
  _OS_IrqDefaultHandler,      // Handler for interrupt 8
  _OS_IrqDefaultHandler,      // Handler for interrupt 9
  _OS_IrqDefaultHandler,      // Handler for interrupt 10
  _OS_IrqDefaultHandler,      // Handler for interrupt 11
  _OS_IrqDefaultHandler,      // Handler for interrupt 12
  _OS_IrqDefaultHandler,      // Handler for interrupt 13
  _OS_IrqDefaultHandler,      // Handler for interrupt 14
  _OS_IrqDefaultHandler,      // Handler for interrupt 15
  _OS_IrqDefaultHandler,      // Handler for interrupt 16
  _OS_IrqDefaultHandler,      // Handler for interrupt 17
  _OS_IrqDefaultHandler,      // Handler for interrupt 18
  _OS_IrqDefaultHandler,      // Handler for interrupt 19
  _OS_IrqDefaultHandler,      // Handler for interrupt 20
  _OS_IrqDefaultHandler,      // Handler for interrupt 21
  _OS_IrqDefaultHandler,      // Handler for interrupt 22
  _OS_IrqDefaultHandler,      // Handler for interrupt 23
  _OS_IrqDefaultHandler,      // Handler for interrupt 24
  _OS_IrqDefaultHandler,      // Handler for interrupt 25
  _OS_IrqDefaultHandler,      // Handler for interrupt 26
  _OS_IrqDefaultHandler,      // Handler for interrupt 27
  _OS_IrqDefaultHandler,      // Handler for interrupt 28
  _OS_IrqDefaultHandler,      // Handler for interrupt 29
  _OS_IrqDefaultHandler,      // Handler for interrupt 30
  _OS_IrqDefaultHandler,      // Handler for interrupt 31
  _OS_IrqDefaultHandler,      // Handler for interrupt 32
  _OS_IrqDefaultHandler,      // Handler for interrupt 33
  _OS_IrqDefaultHandler,      // Handler for interrupt 34
  _OS_IrqDefaultHandler,      // Handler for interrupt 35
  _OS_IrqDefaultHandler,      // Handler for interrupt 36
  _OS_IrqDefaultHandler,      // Handler for interrupt 37
  _OS_IrqDefaultHandler,      // Handler for interrupt 38
  _OS_IrqDefaultHandler,      // Handler for interrupt 39
  _OS_IrqDefaultHandler,      // Handler for interrupt 40
  _OS_IrqDefaultHandler,      // Handler for interrupt 41
  _OS_IrqDefaultHandler,      // Handler for interrupt 42
  _OS_IrqDefaultHandler,      // Handler for interrupt 43
  _OS_IrqDefaultHandler,      // Handler for interrupt 44
  _OS_IrqDefaultHandler,      // Handler for interrupt 45
  _OS_IrqDefaultHandler,      // Handler for interrupt 46
  _OS_IrqDefaultHandler,      // Handler for interrupt 47
  _OS_IrqDefaultHandler,      // Handler for interrupt 48
  _OS_IrqDefaultHandler,      // Handler for interrupt 49
  _OS_IrqDefaultHandler,      // Handler for interrupt 50
  _OS_IrqDefaultHandler,      // Handler for interrupt 51
  _OS_IrqDefaultHandler,      // Handler for interrupt 52
  _OS_IrqDefaultHandler,      // Handler for interrupt 53
  _OS_IrqDefaultHandler,      // Handler for interrupt 54
  _OS_IrqDefaultHandler,      // Handler for interrupt 55
  _OS_IrqDefaultHandler,      // Handler for interrupt 56
  _OS_IrqDefaultHandler,      // Handler for interrupt 57
  _OS_IrqDefaultHandler,      // Handler for interrupt 58
  _OS_IrqDefaultHandler,      // Handler for interrupt 59
  _OS_IrqDefaultHandler,      // Handler for interrupt 60
  _OS_IrqDefaultHandler,      // Handler for interrupt 61
  _OS_IrqDefaultHandler,      // Handler for interrupt 62
  _OS_IrqDefaultHandler,      // Handler for interrupt 63
};

#else
static OS_ISR_HANDLER* _papISR[_NUM_INT_SOURCES];
#endif

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

/*********************************************************************
*
*       _ISR_TickHandler()
*       the OS system interrupt, handles OS timer
*/
static void _ISR_TickHandler(void) {
  OS_HandleTick();
}

/*********************************************************************
*
*       OS_IrqDefaultHandler
*
*/
static void _OS_IrqDefaultHandler(void) {
#if OS_DEBUG
  OS_DI();
  while (1) {
  //
  // The NOPs are necessary for bdm debugger,
  // otherwise the bdm debugger locks up
  //
  #ifdef __IAR_SYSTEMS_ICC__
    __asm("NOP");
    __asm("NOP");
  #else
    asm(NOP);
    asm(NOP);
  #endif
  }
#endif
}

#if (OS_ISRHANDLERTABLE_ROM == 0)
  #define _INIT_VECTORS() _InitVectors()

/*********************************************************************
*
*       _InitVectors()
*/

static void _InitVectors(void) {
  int i;
  OS_ISR_HANDLER* pIsrHandler;

  pIsrHandler = _OS_IrqDefaultHandler;
  for (i = 0; i < _NUM_INT_SOURCES; +i++) {
    _papISR[i] = pIsrHandler;
  }
}
#else
  #define _INIT_VECTORS()
#endif


/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_InitHW()
*
*       Initialize the hardware (timer) required for the OS to run.
*       May be modified, if an other timer should be used
*/
void OS_InitHW(void) {
  OS_DI();
  _INIT_VECTORS();
  OS_EnableISR(55, _ISR_TickHandler, 1, 0);
  OS_RestoreI();
}

/*********************************************************************
*
*       Idle loop  (OS_Idle)
*
*       Please note:
*       This is basically the "core" of the idle loop.
*       This core loop can be changed, but:
*       The idle loop does not have a stack of its own, therefore no
*       functionality should be implemented that relies on the stack
*       to be preserved. However, a simple program loop can be programmed
*       (like toggeling an output or incrementing a counter)
*/
void OS_Idle(void) {         /* Idle loop: No task is ready to exec */
  while (1) {
  //
  // The NOPs are necessary for bdm debugger,
  // otherwise the bdm debugger locks up
  //
  #ifdef __IAR_SYSTEMS_ICC__
    __asm("NOP");
    __asm("NOP");
  #else
    asm(NOP);
    asm(NOP);
  #endif
  }
}

/*********************************************************************
*
*       Get time [cycles]
*
*       This routine is required for task-info via OSView or high
*       resolution time maesurement functions.
*       It returns the system time in timer clock cycles.
*/
OS_U32 OS_GetTime_Cycles(void) {
  return (0);
}

/*********************************************************************
*
*       OS_ConvertCycles2us
*
*       Convert Cycles into micro seconds.
*
*       If your clock frequency is not a multiple of 1 MHz,
*       you may have to modify this routine in order to get proper
*       diagonstics.
*
*       This routine is required for profiling or high resolution time
*       measurement only. It does not affect operation of the OS.
*/
OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  OS_USEPARA(Cycles);       // Not used, avoid warning
  return (0);
}

/*********************************************************************
*
*       OS interrupt handler and ISR specific functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_irq_handler
*
*       Detect reason for IRQ and call correspondig service routine.
*       OS_irq_handler is called from OS_ISR_Handler function
*       from the OS library
*/
void OS_irq_handler(int ISRIndex) {
  OS_ISR_HANDLER* pISR;

  #if OS_DEBUG
    if (ISRIndex < 64) {
      while(1) {
        // OS_irq_Handler handles peripheral interrupts only
        __no_operation();
        __no_operation();
      }
    }
  #endif
  pISR = *_papISR[ISRIndex - 64];
  pISR();
}

/*********************************************************************
*
*       OS_EnableISR
*/
OS_ISR_HANDLER* OS_EnableISR(int ISRIndex, OS_ISR_HANDLER* pISRHandler, int Prio, int SubPrio) {
  OS_ISR_HANDLER*  pOldHandler;
  int i;

#if OS_DEBUG
  if ((ISRIndex >= _NUM_INT_SOURCES) || (ISRIndex == 0)) {
    OS_Error(OS_ERR_ISR_INDEX);
    return NULL;
  }
#endif
  OS_DI();
  pOldHandler = _papISR[ISRIndex];       // Get old handler

#if (OS_ISRHANDLERTABLE_ROM == 0)
  _papISR[ISRIndex] = pISRHandler;       // Install new handler
#endif
  //
  // Check if the interrupt level/priority is unique
  //
  for (i = 8; i < _NUM_INT_SOURCES; i++) {
    if (*(char*) (ICR_BASE_ADDR + i) == Prio) {
      OS_Error(OS_ERR_ISR_VECTOR);
      return NULL;
    }
  }
  *(char*)(ICR_BASE_ADDR + ISRIndex) = (Prio << 3) | SubPrio;   // Set IPL register
  if ( ISRIndex < 32) {                   // Set interrupt mask register IMRL, IMRH
    IMRL &= ~((1uL << ISRIndex) | 0x0001);
  } else {
    IMRL &= ~(0x0001uL);                    // IMRL.bit0 has to be reset to enable any interrupts
    IMRH &= ~(1uL << (ISRIndex-32));
  }
  OS_RestoreI();
  return pOldHandler;
}

/*********************************************************************
*
*       OS_DisableISR
*/
void OS_DisableISR(int ISRIndex) {
#if OS_DEBUG
  if (ISRIndex >= _NUM_INT_SOURCES || (ISRIndex == 0)) {
    OS_Error(OS_ERR_ISR_INDEX);
    return;
  }
#endif
  OS_DI();
  if ( ISRIndex < 32) {                   // Reset interrupt mask register
    IMRL |= (1uL << ISRIndex);
  } else {
    IMRH |= (1uL << (ISRIndex - 32));
  }
  *(char*)(ICR_BASE_ADDR + ISRIndex) = 0x00;   // Set IPL register to 0
#if (OS_ISRHANDLERTABLE_ROM == 0)
  _papISR[ISRIndex] = _OS_IrqDefaultHandler;   // Set default handler
#endif
  OS_RestoreI();
}

/*****  EOF  ********************************************************/

